package jtaDiscRack;

import org.enhydra.dods.*;


import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;

/**
 * The main class of DiscRack application.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class JtaDiscRack extends JPanel {
   // Application Title and other stuff. From resource file.
   private static final String appTitle="JtaDiscRack";
   // Application Icon. From resource file.
   private static ImageIcon appIcon;

   /**
    * Checks if java version is >= 1.4, sets application title and icon, as
    * well as logo icon.
    */
   static {
      try {
         String vers = System.getProperty("java.version");
         if (vers.compareTo("1.") < 0) {
            System.out.println("!!!WARNING: Application must be run with a " +
                  "1.4 or higher version VM!!!");
         }

         // Icon
         URL url = ResourceManager.getResource("Icon");
         if (url != null) appIcon = new ImageIcon(url);

         try {
            //UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
         } catch (Exception exc) {
            System.err.println("Error loading L&F: " + exc);
         }
      } catch (Throwable t) {
         System.err.println("uncaught exception: " + t);
         t.printStackTrace();
      }
   }

   // The frame of application
   protected JFrame myFrame;

   protected Hashtable commands;
   protected JMenuBar menubar;
   protected JToolBar toolbar;
   /**
    * Actions defined by the WorkflowAdmin class
    */
   protected Action[] defaultActions;

   private JScrollPane helperJSP=new JScrollPane();
   private JPanel centralPanel=new JPanel();

   //************************** CONSTRUCTOR ***********************
   /** Creates instance of main application class. */
   public JtaDiscRack() {
      super(true);

      setBorder(BorderFactory.createEtchedBorder());
      setLayout(new BorderLayout());

      commands = new Hashtable();
      // Actions defined by the ProcessEditor class
      createActions();
      Action[] actions = getActions();
      for (int i = 0; i < actions.length; i++) {
         Action a = actions[i];
         commands.put(a.getValue(Action.NAME), a);
      }

      menubar = BarFactory.createMenubar("menubar",commands);
      // adding menubar to the main panel
      add(menubar,BorderLayout.NORTH);
      // adding center component
      JPanel helperPanel=new JPanel();
      helperPanel.setLayout(new BorderLayout());

      toolbar=(JToolBar)BarFactory.createToolbar("toolbar",commands);
      helperPanel.add(toolbar,BorderLayout.NORTH);
      helperJSP.setViewportView(centralPanel);
      helperPanel.add(helperJSP,BorderLayout.CENTER);
      //add(centralPanel,BorderLayout.CENTER);
      add(helperPanel,BorderLayout.CENTER);

      myFrame = new JFrame();
      myFrame.setBackground(Color.lightGray);
      myFrame.getContentPane().setLayout(new BorderLayout());
      myFrame.getContentPane().add(this,BorderLayout.CENTER);
      myFrame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
      myFrame.addWindowListener(new AppCloser());
      myFrame.pack();
      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      int xSize=950,ySize=600;
      int xMinus, yMinus;
      if (screenSize.width>xSize) {
         xMinus=screenSize.width-xSize;
      } else {
         xMinus=0;
      }
      if (screenSize.height>ySize) {
         yMinus=screenSize.height-ySize;
      } else {
         yMinus=0;
      }

      myFrame.setBounds(xMinus/2, yMinus/2, screenSize.width - xMinus,
            screenSize.height - yMinus);

      if (appIcon != null) myFrame.setIconImage(appIcon.getImage());
      setTitleUser(null);

      getAction("Logout").setEnabled(false);
      myFrame.show();
      getAction("Login").actionPerformed(null);

   }
   //************************* END OF WORKLISTHANDLER CONSTRUCTOR *****************

   public void setTitleUser (String user) {
      if (user==null) {
         user="no logged user";
      }
      myFrame.setTitle(appTitle+" - "+user);
   }
   //************** APPCLOSER CLASS FOR CLOSING APPLICATION WINDOW ***************
   /**
    * To shutdown when run as an application.
    */
   protected final class AppCloser extends WindowAdapter {
      public void windowClosing(WindowEvent e) {
         getAction("Exit").actionPerformed(null);
      }
   }
   //**** END OF CREATING APPLICATION CLOSER COMPONENT FOR APPLICATION WINDOW ****

   // ************************* GETTING ACTION STUFF *****************************
   protected void createActions () {
      defaultActions=new Action[] {
         new jtaDiscRack.actions.Exit(this),
         new jtaDiscRack.actions.Login(this),
         new jtaDiscRack.actions.Logout(this),
         new jtaDiscRack.actions.Register(this),
      };
   }

   /**
    * Fetch the list of actions supported by this app.
    */
   public Action[] getActions() {
      return defaultActions;
   }

   /**
    * Method to get action corresponding to the given string.
    * @param cmd String representation of editor's action.
    * @return action specified by the string cmd.
    **/
   public Action getAction(String cmd) {
      return (Action)commands.get(cmd);
   }
   // ********************* END OF GETTING ACTION STUFF **************************

   //*********************** END OF EXIT ACTION CLASS *************************

   public JFrame getFrame () {
      return myFrame;
   }

   public String getAppTitle () {
      return appTitle;
   }

   public void setCentralPanel (JPanel p) {
      centralPanel=p;
      helperJSP.setViewportView(centralPanel);
   }

   public static void main (String[] args) {
      try {
         /*
          * Initialize Application configuration file path
          */
    	 String conFilePath = null;
         String userDir=System.getProperty("user.dir");
    	 conFilePath = userDir+File.separator+"conf"+File.separator+"jtaDiscRack.conf";
    	 
    	 if (args.length>0){
        	 String tempPath = args[0];
        	 File tempConfFile = new File(tempPath);
        	 if (tempConfFile.exists()){
        		 conFilePath = tempConfFile.getCanonicalPath(); 
        	 }
         }
    	 
    	 /*
    	  * Initialize JNDI, JOTM and Datasource JNDI context instance
    	  */
    	 org.enhydra.dods.jts.LocalContextFactory.setup("jtaDiscRackDB");
    	 
    	 /*
    	  * Initialize and configure DODS instalce
    	  */
    	 DODS.startup(conFilePath);
    	 
    	 /*
    	  * Start application
    	  */
    	 JtaDiscRack jdr = new JtaDiscRack();  
      } catch (Exception ex) {
         ex.printStackTrace();
         System.exit(0);
      }
   }
}

