/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: Disc.java,v 1.1 2006/02/23 08:47:29 slobodan Exp $
 */

package jtaDiscRack.business.disc;

import jtaDiscRack.business.JtaDiscRackBusinessException;
import jtaDiscRack.business.XaTransactionSupport;
import jtaDiscRack.business.person.Person;
import jtaDiscRack.data.person.PersonDO;
import jtaDiscRack.data.disc.DiscDO;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.dods.builder.generator.query.DataObjectException;
import org.enhydra.dods.exceptions.AssertionDataObjectException;

/**
 * Represents a disc.
 */
public class Disc extends XaTransactionSupport {

	private static int nextHandleNum = -1;
	/*
	 * Disc attribures 
	 */
	private String handle = null;

	private String artist = null;

	private String genre = null;

	private String title = null;

	private int version = -1;

	private boolean isLiked = false;

	private String owner = null;

	/**
	 * The public constructor.
	 */
	public Disc() throws JtaDiscRackBusinessException {
		handle = String.valueOf(nextHandleNum);
		nextHandleNum--;
	}

	/** The protected constructor
	 *
	 * @param theDisc. The data object of the disc.
	 */
	protected Disc(DiscDO theDisc) throws JtaDiscRackBusinessException {
		try {
			handle = theDisc.get_Handle();
			title = theDisc.getTitle();
			artist = theDisc.getArtist();
			isLiked = theDisc.getIsLiked();
			genre = theDisc.getGenre();
			version = theDisc.getVersion();
			owner = theDisc.getOwner().get_Handle();
		} catch (DatabaseManagerException dbme) {
			dbme.printStackTrace();
			throw new JtaDiscRackBusinessException("Error creating Disc", dbme);
		} catch (DataObjectException doe) {
			doe.printStackTrace();
			throw new JtaDiscRackBusinessException("Error creating Disc", doe);
		}
	}

	/**
	 * Gets the object id for the disc
	 *
	 * @return the object id.
	 * @exception JtaDiscRackBusinessException if an error occurs
	 *   retrieving data (usually due to an underlying data layer
	 *   error).
	 */
	public String getHandle() throws JtaDiscRackBusinessException {
		return handle;
	}


   /**
	  * Gets the status of DO
	  *
	  * @return is Do null.
	  */
   public boolean isDONull(){
	   return true;
   }


	/**
	 * Gets the title for the disc
	 *
	 * @return the title.
	 * @exception JtaDiscRackBusinessException if an error occurs
	 *   retrieving data (usually due to an underlying data layer
	 *   error).
	 */
	public String getTitle() throws JtaDiscRackBusinessException {
		return title;
	}

	/**
	 * Gets the version for the disc
	 *
	 * @return the version.
	 * @exception JtaDiscRackBusinessException if an error occurs
	 *   retrieving data (usually due to an underlying data layer
	 *   error).
	 */
	public int getVersion() throws JtaDiscRackBusinessException {
		return version;
	}

	/**
	 * Gets the artist for the disc
	 *
	 * @return the artist.
	 * @exception JtaDiscRackBusinessException if an error occurs
	 *   retrieving data (usually due to an underlying data layer
	 *   error).
	 */
	public String getArtist() throws JtaDiscRackBusinessException {
		return artist;
	}

	/**
	 * Gets the genre for the disc
	 *
	 * @return the genre.
	 * @exception JtaDiscRackBusinessException if an error occurs
	 *   retrieving data (usually due to an underlying data layer
	 *   error).
	 */
	public String getGenre() throws JtaDiscRackBusinessException {
		return genre;
	}

	/**
	 * Gets the preference for the disc
	 *
	 * @return true if like the disc, false if not
	 * @exception JtaDiscRackBusinessException if an error occurs
	 *   retrieving data (usually due to an underlying data layer
	 *   error).
	 */
	public boolean isLiked() throws JtaDiscRackBusinessException {
		return isLiked;
	}

	/**
	 * Sets the title for the disc.
	 *
	 * @param the title.
	 * @exception JtaDiscRackBusinessException if an error occurs
	 *   setting the data (usually due to an underlying data layer
	 *   error).
	 */
	public void setTitle(String title) throws JtaDiscRackBusinessException {
		this.title = title;
	}

	/**
	 * Sets the artist for the disc.
	 *
	 * @param the artist.
	 * @exception JtaDiscRackBusinessException if an error occurs
	 *   setting the data (usually due to an underlying data layer
	 *   error).
	 */
	public void setArtist(String artist) throws JtaDiscRackBusinessException {
		this.artist = artist;
	}

	/**
	 * Sets the genre for the disc.
	 *
	 * @param the genre.
	 * @exception JtaDiscRackBusinessException if an error occurs
	 *   setting the data (usually due to an underlying data layer
	 *   error).
	 */
	public void setGenre(String genre) throws JtaDiscRackBusinessException {
		this.genre = genre;
	}

	/**
	 * Sets the owner for the disc.
	 *
	 * @param the owner.
	 * @exception JtaDiscRackBusinessException if an error occurs
	 *   setting the data (usually due to an underlying data layer
	 *   error).
	 */
	public void setOwner(Person owner) throws JtaDiscRackBusinessException {
		this.owner = owner.getHandle();
	}

	/**
	 * Sets the preference for the disc.
	 *
	 * @param the preference.
	 * @exception JtaDiscRackBusinessException if an error occurs
	 *   setting the data (usually due to an underlying data layer
	 *   error).
	 */
	public void setLiked(boolean isLiked) throws JtaDiscRackBusinessException {
		this.isLiked = isLiked;
	}

	/**
	 * Sets the version for the disc.
	 *
	 * @param the version.
	 * @exception JtaDiscRackBusinessException if an error occurs
	 *   setting the data (usually due to an underlying data layer
	 *   error).
	 */
	public void setVersion(int ver) throws JtaDiscRackBusinessException {
		this.version = ver;
	}

	/**
	 * Commits all changes to the database.
	 *
	 * @exception JtaDiscRackBusinessException if an error occurs
	 *   retrieving data (usually due to an underlying data layer
	 *   error). 
	 */
	public void save() throws JtaDiscRackBusinessException,
			AssertionDataObjectException {

		initTransaction();

		boolean doCommit = true;
		
		DiscDO myDO = null;

		try {
			PersonDO personDO = PersonDO.createExisting(this.owner);

			if (personDO != null) {
				myDO = DiscDO.createExisting(this.handle);

				if (myDO == null) {
					// disc creation
					myDO = DiscDO.createVirgin();
				}
				myDO.setTitle(this.title);
				myDO.setArtist(this.artist);
				myDO.setGenre(this.genre);
				myDO.setIsLiked(this.isLiked);
				myDO.setOwner(personDO);
				myDO.save();
			}
		} catch (AssertionDataObjectException ex) {
			doCommit = false;
			ex.printStackTrace();
			throw new AssertionDataObjectException(
					"Read-only cache: DML opertions not allowed.", ex);
		} catch (Exception ex) {
			doCommit = false;
			ex.printStackTrace();
			throw new JtaDiscRackBusinessException("Error saving disc", ex);
		} finally {
			if (doCommit) {
				commitTransaction();
			} else {
				rollbackTransaction();
			}
		}
		
		
		try{
			if (myDO !=null){
				this.handle = myDO.get_Handle();
				this.version = myDO.get_Version();
			}
		} catch (Exception ex){
			ex.printStackTrace();
		}
	}

	/**
	 * Deletes the disc from the database.
	 *
	 * @exception JtaDiscRackBusinessException if an error occurs
	 *   deleting data (usually due to an underlying data layer
	 *   error).
	 */
	public void delete() throws JtaDiscRackBusinessException,
			AssertionDataObjectException {

		initTransaction();

		boolean doCommit = true;

		try {
			DiscDO myDO = DiscDO.createExisting(this.handle);
			if (myDO != null) {
				myDO.delete();
			}
		} catch (AssertionDataObjectException ex) {
			doCommit = false;
			throw new AssertionDataObjectException(
					"DML opertions not allowed.", ex);
		} catch (Exception ex) {
			doCommit = false;
			throw new JtaDiscRackBusinessException("Error deleting disc", ex);
		} finally {
			if (doCommit) {
				commitTransaction();
			} else {
				rollbackTransaction();
			}
		}
	}
}
