package jtaDiscRack.presentation;

import jtaDiscRack.*;
import jtaDiscRack.presentation.delements.*;
import jtaDiscRack.presentation.dpanels.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import java.net.URL;

/**
 * The dialog for showing objects derived from {@link DPanel} classes. The
 * given DPanel object must have it's owner, which type is a class
 * derived from the {@link DSimpleElement} class.
 * <p> The dialog enables editing of all editable fields contained
 * within given panel and after user presses OK button, the new values
 * contained within edited fields are set to corresponding members of
 * panel's owner.
 *
 * @author Sasa Bojanic
 * @version 1.0
*/
public class DElementDialog extends JDialog {
   private DPanel panelToEdit=new DPanel();
   private JButton buttonOK;
   private JButton buttonCancel;

   private WindowListener wl=new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
         isCanceled=true;
         dispose();
      }
   };

   /** Set to true if cancel hasn't been pressed */
   private boolean isCanceled=false;

   /**
   * Constructs a new modal instance which parent is <code>parent</code> and
   * proceeds a title of dialog.
   *
   * @param parent Parent frame.
   * @param title  Dialog title.
   */
   public DElementDialog (JFrame parent,String title) {
      super(parent,title,true);
      initDialog();
   }

   /**
   * Constructs a new modal instance which parent is <code>parent</code> and
   * proceeds a title of dialog.
   *
   * @param parent Parent dialog.
   * @param title  Dialog title.
   */
   public DElementDialog (JDialog parent,String title) {
      super(parent,title,true);
      initDialog();
   }

   /**
   * @return <tt>true</tt> if cancel hasn't been pressed,
   *         <tt>false</tt> otherwise.
   */
   public boolean isCanceled () {
      return isCanceled;
   }

   /**
   * Displays dialog and adds specified panel to it, along
   * with OK and/or Cancel button. The dialog allows user to edit
   * the panel field(s) that represents corresponding member of
   * it's owner class. After user presses OK, the content of
   * panel field(s) is transfered to the corresponding owner
   * class member.
   *
   * @param elementPanel The panel with visual presentation of it's
   *                     owner members.
   */
   public void editDElement (DPanel elementPanel,boolean hasCancelButton) {

      Container cp=getContentPane();
      cp.remove(panelToEdit);
      panelToEdit=elementPanel;
      cp.add(panelToEdit,0);
      pack();

      if (hasCancelButton) {
         buttonCancel.setVisible(true);
      } else {
         buttonCancel.setVisible(false);
         buttonOK.requestFocus();
      }

      setLocationRelativeTo(getParent());
      pack();
      show();
      try {
         if (panelToEdit instanceof DGroupPanel) {
            if (panelToEdit.getComponent(0).isEnabled()) {
               panelToEdit.getComponent(0).requestFocus();
            } else {
               panelToEdit.getComponent(1).requestFocus();
            }
         }

      } catch (Exception ex) {
         panelToEdit.requestFocus();
      }

   }

   private void initDialog () {
      try {
         JPanel buttonPanel=new JPanel();

         buttonPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
         buttonPanel.setAlignmentY(Component.TOP_ALIGNMENT);

         buttonOK=new JButton("OK");
         URL u = ResourceManager.getResource("OKImage");
         if (u!=null) {
            buttonOK.setIcon(new ImageIcon(u));
         }

         buttonCancel=new JButton("Cancel");
         u = ResourceManager.getResource("CancelImage");
         if (u!=null) {
            buttonCancel.setIcon(new ImageIcon(u));
         }

         buttonPanel.add(buttonOK);
         buttonPanel.add(buttonCancel);

         Container cp=getContentPane();
         cp.setLayout(new BoxLayout(cp,BoxLayout.Y_AXIS));

         cp.add(panelToEdit);
         cp.add(buttonPanel);

         // action listener for confirming
         buttonOK.addActionListener( new ActionListener(){
            public void actionPerformed( ActionEvent ae ){
               if (canApplyChanges()) {
                  applyChanges();
                  dispose();
               }
            }
         });

         // action listener for canceling
         buttonCancel.addActionListener( new ActionListener(){
            public void actionPerformed( ActionEvent ae ){
               isCanceled=true;
               dispose();
            }
         });


         addWindowListener(wl);

         getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
            .put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0,false),"Cancel");
         getRootPane().getActionMap().put("Cancel", new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
               isCanceled=true;
               dispose();
            }
         });

      } catch (Exception e) {
         e.printStackTrace();
      }
      setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
      setResizable(false);
      setLocationRelativeTo(getParent());
      buttonOK.setDefaultCapable(true);
      getRootPane().setDefaultButton(buttonOK);

   }

   public boolean canApplyChanges () {
      if (panelToEdit.checkRequired()) {
         if (!panelToEdit.getOwner().setDODSElements(panelToEdit)) {
            return false;
         } else {
            return true;
         }
      } else {
         return false;
      }
   }

   public void applyChanges () {
      panelToEdit.setElements();
   }

}


